"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fetchUpdates = fetchUpdates;
const tslib_1 = require("tslib");
// TODO #22198
const is_1 = tslib_1.__importDefault(require("@sindresorhus/is"));
const config_1 = require("../../../config");
const logger_1 = require("../../../logger");
const datasource_1 = require("../../../modules/datasource");
const common_1 = require("../../../modules/datasource/common");
const external_host_error_1 = require("../../../types/errors/external-host-error");
const clone_1 = require("../../../util/clone");
const package_rules_1 = require("../../../util/package-rules");
const p = tslib_1.__importStar(require("../../../util/promises"));
const result_1 = require("../../../util/result");
const stats_1 = require("../../../util/stats");
const package_files_1 = require("../package-files");
const lookup_1 = require("./lookup");
async function lookup(packageFileConfig, indep) {
    const dep = (0, clone_1.clone)(indep);
    dep.updates = [];
    if (dep.skipReason) {
        return result_1.Result.ok(dep);
    }
    if (is_1.default.string(dep.depName)) {
        dep.depName = dep.depName.trim();
    }
    dep.packageName ??= dep.depName;
    if (!is_1.default.nonEmptyString(dep.packageName)) {
        dep.skipReason = 'invalid-name';
        return result_1.Result.ok(dep);
    }
    if (dep.isInternal && !packageFileConfig.updateInternalDeps) {
        dep.skipReason = 'internal-package';
        return result_1.Result.ok(dep);
    }
    const { depName } = dep;
    // TODO: fix types
    let depConfig = (0, config_1.mergeChildConfig)(packageFileConfig, dep);
    const datasourceDefaultConfig = await (0, datasource_1.getDefaultConfig)(depConfig.datasource);
    depConfig = (0, config_1.mergeChildConfig)(depConfig, datasourceDefaultConfig);
    depConfig.versioning ??= (0, common_1.getDefaultVersioning)(depConfig.datasource);
    depConfig = (0, package_rules_1.applyPackageRules)(depConfig, 'pre-lookup');
    depConfig.packageName ??= depConfig.depName;
    if (depConfig.ignoreDeps.includes(depName)) {
        // TODO: fix types (#22198)
        logger_1.logger.debug(`Dependency: ${depName}, is ignored`);
        dep.skipReason = 'ignored';
        return result_1.Result.ok(dep);
    }
    if (depConfig.enabled === false) {
        logger_1.logger.debug(`Dependency: ${depName}, is disabled`);
        dep.skipReason = 'disabled';
        return result_1.Result.ok(dep);
    }
    if (!depConfig.datasource) {
        return result_1.Result.ok(dep);
    }
    return stats_1.LookupStats.wrap(depConfig.datasource, async () => {
        return await result_1.Result.wrap((0, lookup_1.lookupUpdates)(depConfig))
            .onValue((dep) => {
            logger_1.logger.trace({ dep }, 'Dependency lookup success');
        })
            .onError((err) => {
            logger_1.logger.trace({ err, depName }, 'Dependency lookup error');
        })
            .catch((err) => {
            if (packageFileConfig.repoIsOnboarded === true ||
                !(err instanceof external_host_error_1.ExternalHostError)) {
                return result_1.Result.err(err);
            }
            const cause = err.err;
            return result_1.Result.ok({
                updates: [],
                warnings: [
                    {
                        topic: 'Lookup Error',
                        message: `${depName}: ${cause.message}`,
                    },
                ],
            });
        })
            .transform((upd) => Object.assign(dep, upd));
    });
}
function createLookupTasks(config, managerPackageFiles) {
    const lookupTasks = [];
    for (const [manager, packageFiles] of Object.entries(managerPackageFiles)) {
        const managerConfig = (0, config_1.getManagerConfig)(config, manager);
        for (const packageFile of packageFiles) {
            const packageFileConfig = (0, config_1.mergeChildConfig)(managerConfig, packageFile);
            if (packageFile.extractedConstraints) {
                packageFileConfig.constraints = {
                    ...packageFile.extractedConstraints,
                    ...config.constraints,
                };
            }
            for (const dep of packageFile.deps) {
                lookupTasks.push(lookup(packageFileConfig, dep).then((result) => ({
                    packageFileName: packageFile.packageFile,
                    manager: managerConfig.manager,
                    result,
                })));
            }
        }
    }
    return lookupTasks;
}
async function fetchUpdates(config, managerPackageFiles) {
    logger_1.logger.debug({ baseBranch: config.baseBranch }, 'Starting package releases lookups');
    const allTasks = createLookupTasks(config, managerPackageFiles);
    const fetchResults = await Promise.all(allTasks);
    const errors = [];
    const deps = {};
    // Separate good results from errors
    for (const { packageFileName, manager, result } of fetchResults) {
        const { val: dep, err } = result.unwrap();
        if (dep) {
            deps[manager] ??= {};
            deps[manager][packageFileName] ??= [];
            deps[manager][packageFileName].push(dep);
        }
        else {
            errors.push(err);
        }
    }
    if (errors.length) {
        p.handleMultipleErrors(errors);
    }
    // Assign fetched deps back to packageFiles
    for (const [manager, packageFiles] of Object.entries(managerPackageFiles)) {
        for (const packageFile of packageFiles) {
            const packageFileDeps = deps[manager]?.[packageFile.packageFile];
            if (packageFileDeps) {
                packageFile.deps = packageFileDeps;
            }
        }
    }
    package_files_1.PackageFiles.add(config.baseBranch, { ...managerPackageFiles });
    logger_1.logger.debug({ baseBranch: config.baseBranch }, 'Package releases lookups complete');
}
//# sourceMappingURL=fetch.js.map